package cn.estsh.i3plus.ext.mes.apiservice.controller.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderCutService;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrderCut;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrderCutDetail;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
/**
 * 裁片工单
 */
@Api("裁片工单信息")
@RestController
@RequestMapping(MesCommonConstant.MES_YANFEN + "/mesWorkOrderCut")
public class MesWorkOrderCutController extends BaseMesController<MesWorkOrderCut>{

    @Autowired
    private IMesWorkOrderCutService workOrderCutService;

    @PostMapping(value = "/generate")
    @ApiOperation(value = "裁片工单生成", notes = "裁片工单生成")
    public ResultBean doGenerateCutWorkOrder(Long[] workOrderIds, String cutCode, String remark, Integer editionNumber) {
        try {

            ValidatorBean.checkNotNull(workOrderIds, "工单不存在");
            ValidatorBean.checkNotNull(cutCode, "裁片方案不存在");

            workOrderCutService.doGenerateCutWorkOrder(Arrays.asList(workOrderIds), cutCode, remark,
                    AuthUtil.getOrganize().getOrganizeCode(), AuthUtil.getSessionUser().getUserName(), editionNumber);
            return ResultBean.success("生成成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/detail/query")
    @ApiOperation(value = "分页查询裁片工单明细", notes = "分页查询裁片工单明细")
    public ResultBean queryWorkOrderCutDetailByPager(MesWorkOrderCutDetail workOrderCutDetail, Pager pager) {
        try {
            workOrderCutDetail.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success("生成成功")
                    .setListPager(workOrderCutService.queryWorkOrderCutDetailByPager(workOrderCutDetail, pager))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/release")
    @ApiOperation(value = "裁片工单发布", notes = "裁片工单发布")
    public ResultBean doReleaseCutWorkOrder(String[] cutWorkOrderNos) {
        try {

            ValidatorBean.checkNotNull(cutWorkOrderNos, "裁片工单不存在");

            workOrderCutService.doReleaseCutWorkOrder(Arrays.asList(cutWorkOrderNos),  AuthUtil.getOrganize().getOrganizeCode(), AuthUtil.getSessionUser().getUserName());
            return ResultBean.success("发布成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/print")
    @ApiOperation(value = "裁片工单打印", notes = "裁片工单打印")
    public ResultBean print(String[] cutWorkOrderNos) {
        try {

            ValidatorBean.checkNotNull(cutWorkOrderNos, "裁片工单不存在");

            return ResultBean.success("打印成功")
                    .setResultObject(workOrderCutService.doPrintCutWorkOrder(Arrays.asList(cutWorkOrderNos),
                            AuthUtil.getOrganize().getOrganizeCode(), AuthUtil.getSessionUser().getUserName()))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/update-print-status")
    @ApiOperation(value = "修改裁片工单打印状态", notes = "修改裁片工单打印状态")
    public ResultBean doUpdatePrintStatus(String[] cutWorkOrderIds) {
        try {

            ValidatorBean.checkNotNull(cutWorkOrderIds, "裁片工单不存在");

            workOrderCutService.doUpdatePrintStatus(Arrays.asList(cutWorkOrderIds),  AuthUtil.getOrganize().getOrganizeCode(), AuthUtil.getSessionUser().getUserName());
            return ResultBean.success("发布成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
